import {observer} from "mobx-react-lite";
import React from "react";
import {EmailModel, EmailStore} from "../../stores/EmailStore";
import {t} from "@lingui/macro";
import {ButtonField, InputField} from "@rx-suite/ui";
import {CurrentUserStore} from "@rx-suite/stores";
import {makeStyles} from "@material-ui/core/styles";
import {errToString} from "@rx-suite/data";
import {useSnackbar} from "notistack";
import {isEmail} from "../../stores/UserStore";
import {FormHelperText} from "@material-ui/core";

type SendTestEmailButtonProps = {
    email: EmailModel
};

const useStyles = makeStyles(theme => ({
    wrapper: {
        display: 'inline-flex',
        alignItems: 'baseline'
    }
}));

export const SendTestEmailButton: React.FunctionComponent<SendTestEmailButtonProps> = observer(({email}) => {
    const {enqueueSnackbar} = useSnackbar();
    const [recipient, setRecipient] = React.useState('');
    const [sendError, setSendError] = React.useState('');
    const [loading, setLoading] = React.useState(false);
    const classes = useStyles();
    React.useEffect(() => {
        if (CurrentUserStore.selectedModel?.email) {
            setRecipient(curr => curr || CurrentUserStore.selectedModel?.email as string);
        }
    }, [CurrentUserStore.selectedModel?.email]);
    const doSubmit = React.useCallback(async () => {
        try {
            setLoading(true);
            setSendError('');
            await EmailStore.transportLayer.fetch('/test', {
                method: "POST",
                headers: {
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({
                    id: email.getId(),
                    address: email.viewModel.address,
                    password: email.viewModel.password,
                    server: email.viewModel.server,
                    port: email.viewModel.port,
                    auth: email.viewModel.auth,
                    to: recipient
                })
            });
            enqueueSnackbar(t`The email was accepted by the email server and should arrive soon.`, {variant: 'success'});
        } catch (e) {
            const errorMessage = t`Error sending test email: ` + errToString(e);
            enqueueSnackbar(errorMessage, {variant: 'error'});
            setSendError(errorMessage);
        } finally {
            setLoading(false);
        }
    }, [recipient, setLoading, setSendError, enqueueSnackbar, email]);
    const isValidEmail = isEmail(recipient);
    return <>
        <div className={classes.wrapper}>
            <ButtonField text={t`Send A Test Email`} onClick={doSubmit}
                         error={isValidEmail ? undefined : t`Must be a valid email`}
                         disabled={!recipient || !isValidEmail}
                         loading={loading}/>
            <InputField label={t`Test Email Recipient`} value={recipient} disabled={loading}
                        onChange={e => setRecipient(e.target.value)}/>
        </div>
        {sendError && <FormHelperText error>{sendError}</FormHelperText>}
    </>
});