import * as React from "react";
import {SyntheticEvent} from "react";
import {observer} from "mobx-react-lite";
import {t, Trans} from "@lingui/macro";
import {useSnackbar} from "notistack";
import {ComponentPermissionForm} from "../../components/component_permissions/ComponentPermissionForm";
import {FormProps, ModelInput, ModelProvider, PaddedPaper, ReadonlyProvider} from "@rx-suite/ui";
import {Button, LinearProgress} from "@material-ui/core";
import Typography from "@material-ui/core/Typography";
import {useHistory} from "react-router-dom";
import {RxModuleComponentModel, RxModuleComponentStore} from "../../stores/RxModuleComponentStore";
import {Permission, RoleModel} from "../../stores/RoleStore";
import {RolePermissionModel} from "../../stores/RolePermissionStore";
import {LinkButton} from "@rx-suite/ui/build";
import {Edit} from "@material-ui/icons";
import {CurrentUserModel, CurrentUserStore} from "@rx-suite/stores/build";

type RolesFormProps = {
    model: RoleModel
} & Omit<FormProps, 'model'>

export const RolesForm: React.FunctionComponent<RolesFormProps> = observer(({model, readOnly}) => {
    let history = useHistory();

    const {enqueueSnackbar} = useSnackbar();
    const apps = (RxModuleComponentStore.getAll() as RxModuleComponentModel[]).filter(app => ['RxModuleComponent', 'RxUniverseApiComponent'].includes(app.type));

    const permissions = model.permissions || [];

    if (RxModuleComponentStore.isLoading && permissions.length === 0) {
        return <LinearProgress color={'secondary'}/>;
    }

    const createRole = (e: SyntheticEvent) => {
        e.preventDefault();
        processDataChange('/create_with_rules');
    };
    const updateRole = (e: SyntheticEvent) => {
        e.preventDefault();
        processDataChange('/update_with_rules');
    };

    const processDataChange = async (endpoint: string) => {
        if (!model.viewModel.isValid || !permissions.every((permission: Permission) => permission.rolePermission.viewModel.isValid)) {
            enqueueSnackbar(<Trans>Error saving record</Trans>, {variant: "error"});
            return;
        }

        model.viewModel.submit();

        const jsonPayload = {
            roleModel: model.asJson,
            rolePermissions: permissions.map((permission: Permission) => {
                permission.rolePermission.viewModel.submit();
                return permission.rolePermission.asJson;
            })
        };

        const response = await model.sendDataWithRules(endpoint, jsonPayload);

        if (response.ok) {
            enqueueSnackbar(<Trans>Record saved</Trans>, {variant: "success"});

            response.json().then((data: { role: RoleModel, rolePermissions: Array<RolePermissionModel> }) => {
                history.push(`/roles/${data.role.id}/edit`);
            })

            // you *must* be logged in to edit this form, so inform typescript it's ok
            // If you've somehow saved the roles permission form without being logged in we want a crash - that's a bad bug and we want it to be obvious
            if ((CurrentUserStore.selectedModel!.role! as { id: string }).id.toString() === model.getId().toString()) {
                CurrentUserStore.refreshData();
            }
        } else {
            enqueueSnackbar(<Trans>Error saving record</Trans>, {variant: "error"});
        }
    };
    const updateLink = model.store.updateLink(model.getId());
    const user = CurrentUserStore.selectedModel as CurrentUserModel;
    return (
        <PaddedPaper>
            <Typography variant={'h4'}>{t`Role Information`}</Typography>
            <ReadonlyProvider value={!!readOnly}>
                <ModelProvider value={model}>
                    <ModelInput name={"name"}/>
                    <Typography variant={'h4'}>{t`Role Permissions`}</Typography>
                    <ComponentPermissionForm permissions={permissions} apps={apps}/>
                    {readOnly ? (updateLink && (user && model.store.permissionTarget && user.can(model.store.permissionTarget, 'update')) &&
                        <LinkButton to={updateLink} color={"secondary"} noGutter><Edit/> {t`Edit`}
                        </LinkButton>) :
                        <Button variant="contained" color="primary" onClick={model.isNew ? createRole : updateRole}>
                            {model.isNew ? t`Create Role` : t`Update Role`}
                        </Button>}
                </ModelProvider>
            </ReadonlyProvider>
        </PaddedPaper>
    )
});