import {attribute, Model, model, Transport} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {action, computed} from "mobx";
import semver from 'semver';
import {RxModulePackageModel, RxModulePackageStore} from "./RxModulePackageStore";

@model({
    transportLayer: new Transport("/api/components", "component"),
    permissionTarget: "Component",
    routingInfo: {
        list: '/apps?tab=installed',
        create: '/apps/create',
        read: '/apps/:id/read',
        update: '/apps/:id/edit'
    },
})
export class RxModuleComponentModel extends Model {
    @attribute({
        label: t`Type`,
        description: t`Rx-Hub will select an appropriate communication protocol based on the API type`,
        validation: yup.string().required(t`Type is a required field`)
    })
    type: string = 'RxModuleComponent';

    @attribute({autoForm: false, autoTable: false})
    icon_url = "";

    @attribute({
        label: t`IP Address or Network Identifier`,
        description: t`This can be a DNS name or an IP address`,
        validation: yup.string()
    })
    ip_address: string = '';

    @attribute({
        label: t`Port`,
        description: t`The port the app is listening on`,
        validation: yup.number().min(0, t`Port must be a number greater than 0`)
    })
    port: number = 3000;

    @attribute({
        label: t`Name`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    module_name: string = '';

    @attribute({
        label: t`Module ID`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    module_id: string = '';

    @attribute({
        label: t`Module Description`,
        validation: yup.string().ensure(),
        autoForm: false
    })
    description: string = '';

    @attribute({
        label: t`Module Version`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    version: string = '';

    @attribute({
        label: t`Reachable`,
        validation: yup.boolean().nullable(),
        autoForm: false
    })
    reachable: boolean = false;

    @attribute({
        label: t`Updating`,
        validation: yup.boolean().nullable(),
        autoForm: false
    })
    is_updating: boolean = false;

    @attribute({
        label: t`Email Address`,
        validation: yup.number().nullable()
    })
    email_id: string | null = null

    @attribute({
        label: t`Last Update Error`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    last_update_error: string = '';

    @attribute({
        label: t`Update Status`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    update_status: string = '';

    @attribute({
        label: t`Protocol`,
        description: t`Whether the module uses SSL or not`,
        validation: yup.string().nullable(),
        autoForm: false
    })
    protocol: string = '';

    @attribute({autoForm: false, autoTable: false})
    is_api: string = '';

    @action
    async update() {
        return (await (await this.store.transportLayer.fetch(`/${this.module_id}/update`)).json()) as { error: boolean, error_details?: string };
    }

    @action
    async stop() {
        return this.store.transportLayer.fetch(`/${this.module_id}/stop`, { method: "PUT" });
    }

    @action
    async restart() {
        return this.store.transportLayer.fetch(`/${this.module_id}/restart`, { method: "PUT" });
    }

    @computed
    get url() {
        return `http://${this.ip_address}:${this.port}`
    }

    @computed
    get updateText() {
        if (this.is_updating) {
            return this.update_status || t`Pending`
        } else if (this.last_update_error) {
            return this.last_update_error
        } else {
            return t`No Error`;
        }
    }

    fetchUpdateData = () => {
        return RxModulePackageStore.get(this.module_id);
    }

    @computed
    get highestAvailableUpdate() {
        const updateInfo = RxModulePackageStore.data.get(this.module_id) as RxModulePackageModel | undefined;
        if (!updateInfo) {
            return false;
        }
        return updateInfo.highestAvailableUpdate;
    }

    @computed
    get updateAvailable() {
        const highestVersion = this.highestAvailableUpdate;
        if (!highestVersion) {
            return false;
        } else if (!this.version) {
            return true;
        } else {
            return semver.gt(highestVersion, this.version);
        }
    }

    @computed get isRxModule() {
        return this.type === 'RxModuleComponent'
    }

    @computed get usesIcon() {
        return this.isRxModule && !this.is_api;
    }
}

export const RxModuleComponentStore = RxModuleComponentModel.store;