import {attribute, DataStore, Model, Types} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {model} from "@rx-suite/data/build";
import moment, {Moment} from "moment";
import {action, computed} from "mobx";
import uuid from "uuid";

export type ScheduleRecurrenceEvery = 'day' | 'week' | 'month' | 'year';
export type ScheduleRecurrenceWeekday =
    'sunday'
    | 'monday'
    | 'tuesday'
    | 'wednesday'
    | 'thursday'
    | 'friday'
    | 'saturday';
export type ScheduleRecurrenceWeekdayNumeric = '0' | '1' | '2' | '3' | '4' | '5' | '6';
export type ScheduleRecurrenceMonth =
    'january'
    | 'february'
    | 'march'
    | 'april'
    | 'may'
    | 'june'
    | 'july'
    | 'august'
    | 'september'
    | 'october'
    | 'november'
    | 'december';
export type ScheduleRecurrenceOn = ScheduleRecurrenceWeekday[] | {
    [day in ScheduleRecurrenceWeekday]?: number | number[]
};
export type ScheduleRecurrenceDayPeriod = {
    [day in ScheduleRecurrenceWeekdayNumeric]?: number[]
};

// See https://github.com/rossta/montrose#usage for full recurrence_rule info
export type Schedule = {
    id: string,
    start_date: string | Moment, // Required, must be input by user - when to start this shift.
    end_date?: string | Moment | null, // When to stop the recurrence, if desired
    shift_length: number | null, // How long (in seconds) the shift lasts
    recurrence_rule: {
        every: ScheduleRecurrenceEvery, // required, the unit of time to repeat the event
        interval: number | null, // required, how many of .every to wait for recurrences.  Often though not necessarily 1

        on?: ScheduleRecurrenceOn, // on: [array,of,day,names] means trigger on those days (often used with every: 'weekly'). on {dayname: [n]} means on the nth day of the month ({friday: [13, 14]} means on Friday the 13th and Friday the 14th)
        day?: ScheduleRecurrenceDayPeriod // day: {weekday: [n]} means trigger on the nth day in the period.  To be clear: every: 'month', on: {friday: 1} means trigger on Friday the 1st.  Most months do not have a friday the 1st.
        // every: 'month', day:{friday:[1]} means trigger on the first friday of the month - which there will always be one of.
        // every: 'month, on: {'friday': [1,2] means trigger on Friday the 1st or Friday the 2nd - this will only trigger at most once a month since you can't have both.  every: 'month', day:{friday:[1,2]} will trigger twice a month, one week apart.

        mday?: number | number[] // the nth day of the month.  Similar to using day in the {weekday: number} flavour but day-of-the-week agnostic
    }
}

export class CustomShiftStore<T extends Model> extends DataStore<T> {
    async getScheduleEvents(startDate: Date, endDate: Date) {
        startDate.setHours(0, 0, 0, 0);
        endDate.setHours(23, 59, 59, 999)
        const resp = await this.transportLayer.fetch(`/instances?start=${startDate.toISOString()}&end=${endDate.toISOString()}`);
        const respBody = await this.transportLayer.parseJson(resp);
        return respBody.models.map((modelDef: { shift: ShiftModel; dates: { start: string, end: string }[]; }) => {
            return {
                shift: ShiftStore.mapDataToIds([modelDef.shift])[0],
                dates: modelDef.dates.map(d => ({start: moment(d.start), end: moment(d.end)}))
            }
        }) as {
            shift: ShiftModel,
            dates: { start: moment.Moment, end: moment.Moment }[]
        }[];
    }

    async getPreviewEvents(startDate: Date, endDate: Date, shiftSchedules: Array<Schedule>) {
        if (!shiftSchedules || shiftSchedules.length === 0) {
            return [];
        }
        startDate.setHours(0, 0, 0, 0);
        endDate.setHours(23, 59, 59, 999)
        const resp = await this.transportLayer.fetch(`/preview`, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json'
            },
            body: JSON.stringify({
                start: startDate.toISOString(),
                end: endDate.toISOString(),
                shift_schedules: shiftSchedules
            })
        });
        const respBody = await this.transportLayer.parseJson(resp);
        return respBody.dates.map((d: { start: string, end: string }) => {
            return {start: moment(d.start), end: moment(d.end)};
        }) as { start: moment.Moment, end: moment.Moment }[];
    }

    @computed get lastWebsocketUpdate() {
        return Math.max(...this.models.map(m => m.actionCableTimeStamp || 0));
    }

    @computed get hasPendingChanges() {
        return this.models.some(m => m.viewDirty);
    }
}

@model({
    transportLayer: "/api/shifts",
    permissionTarget: "Shift",
    routingInfo: {
        list: '/shifts',
        create: '/shifts/create',
        read: '/shifts/:id/read',
        update: '/shifts/:id/edit'
    },
    storeClass: CustomShiftStore
})
export class ShiftModel extends Model {
    types: Types = {
        shift_schedules: 'json'
    };

    @attribute({
        label: t`Name`,
        description: t`The name of the shift`,
        validation: yup.string().required(t`Name is a required field`)
    })
    name: string = '';

    @attribute({
        label: t`Color`,
        description: t`The color used to distinguish the shift in the calendar`,
        validation: yup.string().nullable()
    })
    color: string = '#3BCADD';

    @attribute({
        autoTable: false, autoForm: false, validation: yup.array().of(yup.object().shape({
            start_date: yup.date().typeError(t`Required: Valid Start Date`).required(t`Start Date is required`),
            end_date: yup.date().nullable().typeError(t`Invalid End Date`),
            shift_length: yup.number().typeError(t`Required: Number > 0`).required(t`Required: Number > 0`).min(1),
            recurrence_rule: yup.object().shape({
                every: yup.string().oneOf(['day', 'week', 'month', 'year']),
                interval: yup.number().typeError(t`Required: Number > 0`).required(t`Required: Number > 0`).min(1),
            })
        }))
    })
    shift_schedules: Array<Schedule> = [];

    @action
    updateFromJson(json: Model, isClone?: boolean) {
        super.updateFromJson(json, isClone);
        this.shift_schedules = this.shift_schedules.map(schedule => {
            return {
                ...schedule,
                start_date: moment(schedule.start_date),
                end_date: schedule.end_date ? moment(schedule.end_date) : undefined,
            }
        });
    }

    @action.bound
    addSchedule() {
        this.viewModel.shift_schedules = [...(this.viewModel.shift_schedules || []), {
            id: uuid(),
            start_date: moment(),
            shift_length: 3600, // 1 hour
            recurrence_rule: {
                every: 'day',
                interval: 1
            }
        }]
    }

    @action.bound
    handleScheduleChange(index: number, newValue: Schedule) {
        this.viewModel.shift_schedules = [...this.viewModel.shift_schedules.slice(0, index),
            newValue,
            ...this.viewModel.shift_schedules.slice(index + 1)];
        this.validate();
    }
}

export const ShiftStore = ShiftModel.store as unknown as CustomShiftStore<ShiftModel>;
