# == Schema Information
#
# Table name: components
#
#  id                :bigint           not null, primary key
#  description       :string
#  has_token         :boolean
#  ip_address        :string
#  is_api            :boolean
#  is_updating       :boolean
#  last_update_error :string
#  module_name       :string
#  port              :integer
#  protocol          :string           default("http")
#  reachable         :boolean
#  token             :string
#  type              :string
#  update_status     :string
#  version           :string
#  created_at        :datetime         not null
#  updated_at        :datetime         not null
#  email_id          :bigint
#  module_id         :string
#
# Indexes
#
#  index_components_on_email_id             (email_id)
#  index_components_on_module_id            (module_id) UNIQUE
#  index_components_on_port_and_ip_address  (port,ip_address) UNIQUE
#
# Foreign Keys
#
#  fk_rails_...  (email_id => emails.id) ON DELETE => nullify
#
class RxUniverseApiComponent < Component
  before_save :set_module_values

  def set_module_values
    self.module_id = "rx_universe_api"
    self.module_name = "Rx-Universe API"
    self.description = "Manages Communication with Rx-Universe"
    self.is_api = true
  end

  def oauth2_token_url
    "#{index}/oauth2/token"
  end

  def oauth2_refresh_url
    "#{index}/oauth2/refresh"
  end

  def broadcast_local_name
    Component.model_name.element.to_sym # single table inheritance may mean the actioncable broadcast name needs configuration
  end

  def to_partial_path
    Component._to_partial_path
  end

  def self._to_partial_path
    Component._to_partial_path
  end

  def ping
    status = RxModule::Communicator::Rxuniverse.api_info
    last_update_error = status[:last_update] || {}

    updates = {
      version: status[:version],
      reachable: true,
      is_updating: last_update_error[:applying] || false,
      last_update_error: last_update_error[:error] || nil
    }

    if status.has_key?(:ssl)
      updates[:protocol] = status[:ssl] ? "https" : "http"
    end
    update(updates)
    permission = component_permissions.where(target_id: "update", action_id: "apply").first_or_create
    permission[:target_display_name] = "Updates"
    permission[:action_display_name] = "Apply"
    permission.save
    component_permissions.where.not(id: permission.id).destroy_all
  rescue => err
    Rails.logger.warn { {msg: "Component may be offline or returning invalid responses", component: as_json} }
    update(reachable: false)
  end

  def endpoint_authorization(endpoint)
    if RxModule::Authorization.current_user
      user = User.find(RxModule::Authorization.current_user[:id])
      user.rxuniverse_service&.current_token!
    end
  end

  def upgrade!
    mark_downloading_update
    resp = RxModule::Communicator.call_module_method(module_id, "/api/updates/apply", :post, callback_url: RxModule::Communicator.resolve_endpoint("rx_hub", "/api/components/rxuniverse_api_startup"))
    if resp[:error]
      mark_update_error(resp[:error])
      raise resp[:error]
    else
      mark_update_applying
    end
  rescue => err
    mark_update_error(err.message)
    raise err
  end
end
